# Generated by roxygen2: do not edit by hand

S3method("[",pkg_resolution_result)
S3method("[",pkg_scan_deps)
S3method("[",pkg_solution_result)
S3method("[",pkg_sysreqs_check_result)
S3method("[",pkgplan_downloads)
S3method(format,package_build_error)
S3method(format,package_packaging_error)
S3method(format,package_uncompress_error)
S3method(format,pkg_name_check)
S3method(format,pkg_name_check_basics)
S3method(format,pkg_name_check_sentiment)
S3method(format,pkg_name_check_urban)
S3method(format,pkg_name_check_wikipedia)
S3method(format,pkg_name_check_wiktionary)
S3method(format,pkg_scan_deps)
S3method(format,pkg_solution_failures)
S3method(format,pkg_solution_result)
S3method(format,pkg_sysreqs_check_result)
S3method(format,pkgplan_lp_problem)
S3method(format,remote_ref)
S3method(print,package_build_error)
S3method(print,package_packaging_error)
S3method(print,package_uncompress_error)
S3method(print,pkg_name_check)
S3method(print,pkg_name_check_basics)
S3method(print,pkg_name_check_sentiment)
S3method(print,pkg_name_check_urban)
S3method(print,pkg_name_check_wikipedia)
S3method(print,pkg_name_check_wiktionary)
S3method(print,pkg_scan_deps)
S3method(print,pkg_solution_failures)
S3method(print,pkg_solution_result)
S3method(print,pkg_sysreqs_check_result)
S3method(print,pkginstall_result)
S3method(print,pkgplan_lp_problem)
S3method(print,snapshot)
S3method(snapshot,pkg_resolution_result)
export(as_pkg_dependencies)
export(current_config)
export(current_r_platform)
export(default_platforms)
export(ghr)
export(ghrepo)
export(install_package_plan)
export(is_valid_package_name)
export(lib_status)
export(new_pkg_deps)
export(new_pkg_download_proposal)
export(new_pkg_installation_plan)
export(new_pkg_installation_proposal)
export(parse_pkg_ref)
export(parse_pkg_refs)
export(pkg_build)
export(pkg_dep_types)
export(pkg_dep_types_hard)
export(pkg_dep_types_soft)
export(pkg_deps)
export(pkg_download_proposal)
export(pkg_installation_plan)
export(pkg_installation_proposal)
export(pkg_name_check)
export(pkg_rx)
export(repo)
export(scan_deps)
export(sysreqs_check_installed)
export(sysreqs_db_list)
export(sysreqs_db_match)
export(sysreqs_db_update)
export(sysreqs_fix_installed)
export(sysreqs_install_plan)
export(sysreqs_is_supported)
export(sysreqs_list_system_packages)
export(sysreqs_platforms)
importFrom(stats,na.omit)
importFrom(utils,modifyList)
importFrom(utils,untar)
useDynLib(pkgdepends, .registration = TRUE, .fixes = "c_")
