# installed preferred over download

    Code
      sol[, c("type", "package", "version")]
    Output
      # A data frame: 2 x 3
        type      package version
      * <chr>     <chr>   <chr>  
      1 installed pkg1    1.0.0  
      2 installed pkg2    1.0.0  

---

    Code
      sol[, c("type", "package", "version")]
    Output
      # A data frame: 2 x 3
        type      package version
      * <chr>     <chr>   <chr>  
      1 installed pkg1    1.0.0  
      2 installed pkg2    1.0.0  

---

    Code
      sol[, c("type", "package", "version")]
    Output
      # A data frame: 2 x 3
        type     package version
      * <chr>    <chr>   <chr>  
      1 standard pkg1    1.0.0  
      2 standard pkg2    1.0.0  

