# resolve_remote

    Code
      as.list(res[, intersect(names(res), cols)])
    Output
      $package
      [1] "Biobase"
      
      $version
      [1] "1.2.3"
      
      $needscompilation
      [1] FALSE
      
      $repodir
      [1] "src/contrib"
      
      $rversion
      [1] "*"
      
      $platform
      [1] "source"
      
      $priority
      [1] NA
      
      $ref
      [1] "Biobase"
      
      $type
      [1] "standard"
      
      $status
      [1] "OK"
      
      $target
      [1] "src/contrib/Biobase_1.2.3.tar.gz"
      
      $mirror
      [1] "http://127.0.0.1:<port>//packages/<bioc-version>/bioc"
      
      $sources
      $sources[[1]]
      [1] "http://127.0.0.1:<port>//packages/<bioc-version>/bioc/src/contrib/Biobase_1.2.3.tar.gz"
      
      
      $sysreqs
      [1] NA
      
      $deps
      $deps[[1]]
      # A data frame: 9 x 5
        ref          type     package      op    version 
        <chr>        <chr>    <chr>        <chr> <chr>   
      1 ALL          suggests ALL          ""    ""      
      2 BiocGenerics depends  BiocGenerics ">="  "0.27.1"
      3 R            depends  R            ">="  "2.10"  
      4 RUnit        suggests RUnit        ""    ""      
      5 golubEsets   suggests golubEsets   ""    ""      
      6 methods      imports  methods      ""    ""      
      7 tkWidgets    suggests tkWidgets    ""    ""      
      8 tools        suggests tools        ""    ""      
      9 utils        depends  utils        ""    ""      
      
      
      $built
      [1] NA
      
      $repotype
      [1] "bioc"
      
      $direct
      [1] TRUE
      
      $params
      $params[[1]]
      character(0)
      
      
      $metadata
      $metadata[[1]]
                                        RemoteType 
                                        "standard" 
                                      RemotePkgRef 
                                         "Biobase" 
                                         RemoteRef 
                                         "Biobase" 
                                       RemoteRepos 
      "http://127.0.0.1:<port>//packages/<bioc-version>/bioc" 
                                 RemotePkgPlatform 
                                          "source" 
                                         RemoteSha 
                                           "1.2.3" 
      
      

