# uncompress error

    Code
      install_package_plan(plan, lib = tempdir())
    Message
      i Packaging foo 
      x Failed to uncompress foo 
    Condition
      Error:
      ! ! Failed to uncompress foo from .../foo-t.zip'.

