# failure in non-needed package is ignored

    Code
      p$draw()
    Output
      needspak 1.0.0 [new][bld][dl] (<size>)
      \-r-lib/pak 1.0.0 [new][bld][cmp][dl] (unknown size)
      
      r-lib/pak 1.0.0 [new][bld][cmp][dl] (unknown size)
      
      Key:  [new] new | [dl] download | [bld] build | [cmp] compile

