# resolve_remote_git

    Code
      as.list(res)
    Output
      $ref
      [1] "git::https://github.com/r-lib/cli@v3.6.0"
      
      $type
      [1] "git"
      
      $direct
      [1] TRUE
      
      $directpkg
      [1] TRUE
      
      $status
      [1] "OK"
      
      $package
      [1] "cli"
      
      $version
      [1] "3.6.0"
      
      $license
      [1] "MIT + file LICENSE"
      
      $needscompilation
      [1] TRUE
      
      $priority
      [1] NA
      
      $md5sum
      [1] NA
      
      $sha256
      [1] NA
      
      $filesize
      [1] NA
      
      $built
      [1] NA
      
      $platform
      [1] "source"
      
      $rversion
      [1] "*"
      
      $repotype
      [1] NA
      
      $repodir
      [1] "src/contrib"
      
      $target
      [1] "src/contrib/cli_3.6.0_git_4cc6312"
      
      $deps
      $deps[[1]]
      # A data frame: 32 x 5
         ref         type     package     op    version
         <chr>       <chr>    <chr>       <chr> <chr>  
       1 R           Depends  R           ">="  "3.4"  
       2 callr       Suggests callr       ""    ""     
       3 covr        Suggests covr        ""    ""     
       4 crayon      Suggests crayon      ""    ""     
       5 digest      Suggests digest      ""    ""     
       6 glue        Suggests glue        ">="  "1.6.0"
       7 htmltools   Suggests htmltools   ""    ""     
       8 htmlwidgets Suggests htmlwidgets ""    ""     
       9 knitr       Suggests knitr       ""    ""     
      10 mockery     Suggests mockery     ""    ""     
      # i 22 more rows
      
      
      $mirror
      [1] NA
      
      $sources
      $sources[[1]]
      [1] "https://github.com/r-lib/cli"
      
      
      $remote
      $remote[[1]]
      $package
      [1] "cli"
      
      $protocol
      [1] "https"
      
      $host
      [1] "github.com"
      
      $path
      [1] "/r-lib/"
      
      $repo
      [1] "cli"
      
      $commitish
      [1] "v3.6.0"
      
      $ref
      [1] "git::https://github.com/r-lib/cli@v3.6.0"
      
      $type
      [1] "git"
      
      $dotgit
      [1] ""
      
      $url
      [1] "https://github.com/r-lib/cli"
      
      $params
      character(0)
      
      attr(,"class")
      [1] "remote_ref_git" "remote_ref"     "list"          
      
      
      $error
      $error[[1]]
      list()
      
      
      $metadata
      $metadata[[1]]
                                      RemoteType 
                                           "git" 
                                       RemoteUrl 
                  "https://github.com/r-lib/cli" 
                                    RemotePkgRef 
      "git::https://github.com/r-lib/cli@v3.6.0" 
                                       RemoteRef 
                                        "v3.6.0" 
                                       RemoteSha 
      "4cc6312883f2842e948f34a816ce8e846f323a6e" 
      
      
      $extra
      $extra[[1]]
      $extra[[1]]$remotesha
      [1] "4cc6312883f2842e948f34a816ce8e846f323a6e"
      attr(,"protocol")
      [1] 2
      attr(,"filter")
      [1] TRUE
      
      
      
      $dep_types
      $dep_types[[1]]
      [1] "Depends"   "Imports"   "LinkingTo"
      
      
      $params
      $params[[1]]
      character(0)
      
      
      $sysreqs
      [1] ""
      
      $os_type
      [1] NA
      
      $cache_status
      [1] "miss"
      
      attr(,"metadata")
      named list()

