# pkgplan_i_lp_dependencies

    Code
      lp$conds[[3]]
    Output
      $vars
      [1]  7 13 28
      
      $coef
      [1]  1 -1 -1
      
      $op
      [1] "<="
      
      $rhs
      [1] 0
      
      $type
      [1] "dependency"
      
      $note
      $note$wh
      [1] 7
      
      $note$ref
      [1] "rlang"
      
      $note$cand
      [1] 13 28
      
      $note$good_cand
      [1] 13 28
      
      $note$txt
      [1] "hms depends on rlang: version rlang 0.4.0, rlang 0.4.0"
      
      $note$depop
      [1] ""
      
      $note$depver
      [1] ""
      
      

