# draw_solution_tree [plain]

    Code
      r$draw_solution_tree()
    Output
      dplyr 1.0.10 [new][bld][dl] (<size>)
      +-R6 2.5.1 [new][bld][dl] (<size>)
      +-generics 0.1.3 [new][bld][dl] (<size>)
      +-glue 1.6.2 [new][bld][dl] (<size>)
      +-lifecycle 1.0.3 [new][bld][dl] (<size>)
      | +-cli 3.4.1 [new][bld][dl] (<size>)
      | +-glue
      | \-rlang 1.0.6 [new][bld][dl] (<size>)
      +-magrittr 2.0.3 [new][bld][dl] (<size>)
      +-pillar 1.8.1 [new][bld][dl] (<size>)
      | +-cli
      | +-fansi 1.0.3 [new][bld][dl] (<size>)
      | +-glue
      | +-lifecycle
      | +-rlang
      | +-utf8 1.2.2 [new][bld][dl] (<size>)
      | \-vctrs 0.4.2 [new][bld][dl] (<size>)
      |   +-cli
      |   +-glue
      |   \-rlang
      +-rlang
      +-tibble 3.1.8 [new][bld][dl] (<size>)
      | +-fansi
      | +-lifecycle
      | +-magrittr
      | +-pillar
      | +-pkgconfig 2.0.3 [new][bld][dl] (<size>)
      | +-rlang
      | \-vctrs
      +-tidyselect 1.2.0 [new][bld][dl] (<size>)
      | +-cli
      | +-glue
      | +-lifecycle
      | +-rlang
      | +-vctrs
      | \-withr 2.5.0 [new][bld][dl] (<size>)
      \-vctrs
      
      pkgconfig 2.0.3 [new][bld][dl] (<size>)
      
      Key:  [new] new | [dl] download | [bld] build

# draw_solution_tree [ansi]

    Code
      r$draw_solution_tree()
    Output
      [1m[3m[36mdplyr [39m[90m1.0.10[39m[23m[22m [32m[new][bld][dl] [39m[90m(<size>)[39m
      +-R6 [90m2.5.1[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      +-generics [90m0.1.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      +-glue [90m1.6.2[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      +-lifecycle [90m1.0.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-cli [90m3.4.1[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-glue
      | \-rlang [90m1.0.6[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      +-magrittr [90m2.0.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      +-pillar [90m1.8.1[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-cli
      | +-fansi [90m1.0.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-glue
      | +-lifecycle
      | +-rlang
      | +-utf8 [90m1.2.2[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | \-vctrs [90m0.4.2[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      |   +-cli
      |   +-glue
      |   \-rlang
      +-rlang
      +-tibble [90m3.1.8[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-fansi
      | +-lifecycle
      | +-magrittr
      | +-pillar
      | +-[1m[3m[36mpkgconfig [39m[90m2.0.3[39m[23m[22m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-rlang
      | \-vctrs
      +-tidyselect [90m1.2.0[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      | +-cli
      | +-glue
      | +-lifecycle
      | +-rlang
      | +-vctrs
      | \-withr [90m2.5.0[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      \-vctrs
      
      [1m[3m[36mpkgconfig [39m[90m2.0.3[39m[23m[22m [32m[new][bld][dl] [39m[90m(<size>)[39m
      
      Key:  [32m[new][39m new | [32m[dl][39m download | [32m[bld][39m build

# draw_solution_tree [unicode]

    Code
      r$draw_solution_tree()
    Output
      dplyr 1.0.10 [new][bld][dl] (<size>)
      ├─R6 2.5.1 [new][bld][dl] (<size>)
      ├─generics 0.1.3 [new][bld][dl] (<size>)
      ├─glue 1.6.2 [new][bld][dl] (<size>)
      ├─lifecycle 1.0.3 [new][bld][dl] (<size>)
      │ ├─cli 3.4.1 [new][bld][dl] (<size>)
      │ ├─glue
      │ └─rlang 1.0.6 [new][bld][dl] (<size>)
      ├─magrittr 2.0.3 [new][bld][dl] (<size>)
      ├─pillar 1.8.1 [new][bld][dl] (<size>)
      │ ├─cli
      │ ├─fansi 1.0.3 [new][bld][dl] (<size>)
      │ ├─glue
      │ ├─lifecycle
      │ ├─rlang
      │ ├─utf8 1.2.2 [new][bld][dl] (<size>)
      │ └─vctrs 0.4.2 [new][bld][dl] (<size>)
      │   ├─cli
      │   ├─glue
      │   └─rlang
      ├─rlang
      ├─tibble 3.1.8 [new][bld][dl] (<size>)
      │ ├─fansi
      │ ├─lifecycle
      │ ├─magrittr
      │ ├─pillar
      │ ├─pkgconfig 2.0.3 [new][bld][dl] (<size>)
      │ ├─rlang
      │ └─vctrs
      ├─tidyselect 1.2.0 [new][bld][dl] (<size>)
      │ ├─cli
      │ ├─glue
      │ ├─lifecycle
      │ ├─rlang
      │ ├─vctrs
      │ └─withr 2.5.0 [new][bld][dl] (<size>)
      └─vctrs
      
      pkgconfig 2.0.3 [new][bld][dl] (<size>)
      
      Key:  [new] new | [dl] download | [bld] build

# draw_solution_tree [fancy]

    Code
      r$draw_solution_tree()
    Output
      [1m[3m[36mdplyr [39m[90m1.0.10[39m[23m[22m [32m[new][bld][dl] [39m[90m(<size>)[39m
      ├─R6 [90m2.5.1[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      ├─generics [90m0.1.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      ├─glue [90m1.6.2[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      ├─lifecycle [90m1.0.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─cli [90m3.4.1[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─glue
      │ └─rlang [90m1.0.6[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      ├─magrittr [90m2.0.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      ├─pillar [90m1.8.1[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─cli
      │ ├─fansi [90m1.0.3[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─glue
      │ ├─lifecycle
      │ ├─rlang
      │ ├─utf8 [90m1.2.2[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ └─vctrs [90m0.4.2[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │   ├─cli
      │   ├─glue
      │   └─rlang
      ├─rlang
      ├─tibble [90m3.1.8[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─fansi
      │ ├─lifecycle
      │ ├─magrittr
      │ ├─pillar
      │ ├─[1m[3m[36mpkgconfig [39m[90m2.0.3[39m[23m[22m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─rlang
      │ └─vctrs
      ├─tidyselect [90m1.2.0[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      │ ├─cli
      │ ├─glue
      │ ├─lifecycle
      │ ├─rlang
      │ ├─vctrs
      │ └─withr [90m2.5.0[39m [32m[new][bld][dl] [39m[90m(<size>)[39m
      └─vctrs
      
      [1m[3m[36mpkgconfig [39m[90m2.0.3[39m[23m[22m [32m[new][bld][dl] [39m[90m(<size>)[39m
      
      Key:  [32m[new][39m new | [32m[dl][39m download | [32m[bld][39m build

# tree from lockfile

    Code
      plan$draw()
    Output
      pkg3 1.0.0 [new][bld][dl] (<size>)
      \-pkg2 1.0.0 [new][bld][dl] (<size>)
        \-pkg1 1.0.0 [new][bld][dl] (<size>)
      
      Key:  [new] new | [dl] download | [bld] build

# update

    Code
      plan2$draw()
    Output
      pkg1 0.9.0 -> 1.0.0 [upd][bld][dl] (<size>)
      
      Key:  [upd] update | [dl] download | [bld] build

---

    Code
      plan3$draw()
    Output
      pkg1 0.9.0 < 1.0.0 [old]
      
      Key:  [old] outdated

# no emoji

    Code
      emoji("rocket")
    Output
      [1] "[upd]"
    Code
      emoji("sparkles")
    Output
      [1] "[new]"
    Code
      emoji("hand")
    Output
      [1] "[old]"
    Code
      emoji("dl")
    Output
      [1] "[dl]"
    Code
      emoji("builder")
    Output
      [1] "[bld]"
    Code
      emoji("wrench")
    Output
      [1] "[cmp]"
    Code
      emoji("pkg")
    Output
      [1] "pkg"
    Code
      emoji("pkgs")
    Output
      [1] "pkgs"
    Code
      emoji("foobar")
    Output
      [1] ""

# emoji

    Code
      emoji("rocket")
    Output
      [1] "🚀"
    Code
      emoji("sparkles")
    Output
      [1] "✨"
    Code
      emoji("hand")
    Output
      [1] "✋"
    Code
      emoji("dl")
    Output
      [1] " ⬇"
    Code
      emoji("builder")
    Output
      [1] "👷"
    Code
      emoji("wrench")
    Output
      [1] "🔧"
    Code
      emoji("pkg")
    Output
      [1] "📦"
    Code
      emoji("pkgs")
    Output
      [1] "📦"
    Code
      emoji("foobar")
    Output
      [1] ""

# emo_builder

    Code
      emo_builder(5)
    Output
      [1] "👷" "👷" "👷" "👷" "👷"

---

    Code
      emo_builder(5)
    Output
      [1] "👷🏻" "👷🏼" "👷🏽" "👷🏾" "👷🏿"

---

    Code
      emo_builder(5)
    Output
      [1] "👷🏽‍♂️" "👷🏾‍♀️" "👷🏾‍♂️" "👷🏿‍♀️" "👷🏿‍♂️"

